/** 
 *  Hyper Operating System V4 Advance
 *
 * @file  pol_sem.c
 * @brief %jp{Z}tH̊l(|[O)}%en{Acquire Semaphore Resource(Polling)}
 *
 * Copyright (C) 1998-2006 by Project HOS
 * http://sourceforge.jp/projects/hos/
 */



#include "core/core.h"
#include "object/semobj.h"



#if _KERNEL_SPT_POL_SEM


#if _KERNEL_SPT_TWAI_SEM && (_KERNEL_OPT_CODE_SIZE <= _KERNEL_OPT_SPEED)	/* %jp{twai_semŁATCYDȂ} */

/** %jp{Z}tH̊l(|[O)}%en{Acquire Semaphore Resource(Polling)}
 * @param  semid    %jp{lΏۂ̃Z}tHIDԍ}%en{ID number of the semaphore from which resource is acquired}
 * @retval E_OK     %jp{I}%en{Normal completion}
 * @retval E_ID     %jp{sIDԍ(semids邢͎gpłȂ)}%en{Invalid ID number(semid is invalid or unusable)}
 * @retval E_NOEXS  %jp{IuWFNg(ΏۃZ}tHo^)}%en{Non-existant object(specified semaphore is not registerd)}
 * @retval E_TMOUT  %jp{|[Os}%en{Polling failure}
 */
ER pol_sem(ID semid)
{
	/* %jp{pol_semtwai_semƋʏƂ} */
	return _kernel_wai_sem(semid, TMO_POL);
}

#else

/** %jp{Z}tH̊l(|[O)}%en{Acquire Semaphore Resource(Polling)}
 * @param  semid    %jp{lΏۂ̃Z}tHIDԍ}%en{ID number of the semaphore from which resource is acquired}
 * @retval E_OK     %jp{I}%en{Normal completion}
 * @retval E_ID     %jp{sIDԍ(semids邢͎gpłȂ)}%en{Invalid ID number(semid is invalid or unusable)}
 * @retval E_NOEXS  %jp{IuWFNg(ΏۃZ}tHo^)}%en{Non-existant object(specified semaphore is not registerd)}
 * @retval E_TMOUT  %jp{|[Os}%en{Polling failure}
 */
ER pol_sem(ID semid)
{
	_KERNEL_T_SEMCB_PTR  semcb;
	_KERNEL_SEM_T_SEMCNT semcnt;
	ER                   ercd;
	
	/* %jp{ID ̃`FbN} */
#if _KERNEL_SPT_POL_SEM_E_ID
	if ( !_KERNEL_SEM_CHECK_SEMID(semid) )
	{
		return E_ID;	/* %jp{sIDԍ}%en{Invalid ID number} */
	}
#endif
	
	_KERNEL_ENTER_SVC();		/* %jp{T[rXR[ɓ}%en{enter service-call} */
	
	/* %jp{IuWFNg݃`FbN} */
#if _KERNEL_SPT_POL_SEM_E_NOEXS
	if ( !_KERNEL_SEM_CHECK_EXS(semid) )
	{
		_KERNEL_LEAVE_SVC();	/* %jp{T[rXR[o}%en{leave service-call} */
		return E_NOEXS;			/* %jp{IuWFNg}%en{Non-existant object} */
	}
#endif

	/* %jp{Z}tHRg[ubN擾} */
	semcb = _KERNEL_SEM_ID2SEMCB(semid);
	
	/* %jp{Z}tHJE^擾} */
	semcnt = _KERNEL_SEM_GET_SEMCNT(semcb);

	/* %jp{`FbN} */
	if ( semcnt > 0 ) 
	{
		/* %jp{Z}tHl} */
		_KERNEL_SEM_SET_SEMCNT(semcb, semcnt - 1); 		/* %jp{Z}tH̊l} */
		ercd = E_OK;
	}
	else
	{
		ercd = E_TMOUT;
	}
	
	
	_KERNEL_LEAVE_SVC();	/* %jp{T[rXR[o}%en{leave service-call} */
	
	return ercd;
}

#endif


#else	/* _KERNEL_SPT_POL_SEM */


#if _KERNEL_SPT_POL_SEM_E_NOSPT

/** %jp{Z}tH̊l(|[O)}%en{Acquire Semaphore Resource(Polling)}
 * @param  semid    %jp{lΏۂ̃Z}tHIDԍ}%en{ID number of the semaphore from which resource is acquired}
 * @retval E_NOSPT  %jp{T|[g@\}%en{Unsupported function}
 */
ER wai_sem(ID semid)
{
	return E_NOSPT;
}

#endif


#endif	/* _KERNEL_SPT_POL_SEM */



/* end of file */
